@@ -1,18 +1,10 @@
 """
 For internal use only.
 """
+from __future__ import annotations
+
 import re
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Dict,
-    Iterable,
-    Optional,
-    Type,
-    TypeVar,
-    cast,
-)
+from typing import TYPE_CHECKING, Any, Callable, Iterable, TypeVar, cast
 
 from prompt_toolkit.document import Document
 from prompt_toolkit.formatted_text import to_formatted_text
@@ -65,8 +57,8 @@ def has_unclosed_brackets(text: str) -> bool:
 
 
 def get_jedi_script_from_document(
-    document: Document, locals: Dict[str, Any], globals: Dict[str, Any]
-) -> "Interpreter":
+    document: Document, locals: dict[str, Any], globals: dict[str, Any]
+) -> Interpreter:
     import jedi  # We keep this import in-line, to improve start-up time.
 
     # Importing Jedi is 'slow'.
@@ -89,7 +81,7 @@ def get_jedi_script_from_document(
         # Workaround Jedi issue #514: for https://github.com/davidhalter/jedi/issues/514
         return None
     except KeyError:
-        # Workaroud for a crash when the input is "u'", the start of a unicode string.
+        # Workaround for a crash when the input is "u'", the start of a unicode string.
         return None
     except Exception:
         # Workaround for: https://github.com/jonathanslenders/ptpython/issues/91
@@ -154,7 +146,7 @@ def if_mousedown(handler: _T) -> _T:
     by the Window.)
     """
 
-    def handle_if_mouse_down(mouse_event: MouseEvent) -> "NotImplementedOrNone":
+    def handle_if_mouse_down(mouse_event: MouseEvent) -> NotImplementedOrNone:
         if mouse_event.event_type == MouseEventType.MOUSE_DOWN:
             return handler(mouse_event)
         else:
